"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var fs = require("fs");

var reg = require("./reg");

var promisify = require("util").promisify || require("util.promisify");

var writeFile = promisify(fs.writeFile);

function identifiers() {
  return _identifiers.apply(this, arguments);
}

function _identifiers() {
  _identifiers = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee() {
    var database, res, got;
    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            database = Object.assign({
              codepage: {},
              encoding: {}
            }, function () {
              try {
                return require("./charset.json");
              } catch (ex) {//
              }
            }());
            _context.prev = 1;
            got = require("got");
            _context.next = 5;
            return got("https://docs.microsoft.com/en-us/windows/desktop/intl/code-page-identifiers");

          case 5:
            res = _context.sent;
            _context.next = 11;
            break;

          case 8:
            _context.prev = 8;
            _context.t0 = _context["catch"](1);
            return _context.abrupt("return", database);

          case 11:
            res.body.replace(/<tr>\s+<td>\s*(\d+)\s*<\/td>\s*<td>\s*(?:[xX]-)?(.+?)\s*<\/td>/g, function (s, codepage, encoding) {
              codepage = +codepage;
              encoding = canonicalizeEncoding(encoding);
              database.encoding[String(codepage)] = encoding;
              database.codepage[encoding] = codepage;
            });
            return _context.abrupt("return", database);

          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this, [[1, 8]]);
  }));
  return _identifiers.apply(this, arguments);
}

function canonicalizeEncoding(encoding) {
  // Canonicalize encoding name: strip all non-alphanumeric chars
  return encoding.replace(/[\W_]+/g, "").toLowerCase();
}

function codepage(_x) {
  return _codepage.apply(this, arguments);
}

function _codepage() {
  _codepage = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee2(encoding) {
    var codepage;
    return _regenerator.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return reg.query("HKCR/MIME/Database/Codepage");

          case 2:
            codepage = _context2.sent;
            Object.keys(codepage).forEach(function (cp) {
              var name = codepage[cp].WebCharset || codepage[cp].BodyCharset;
              encoding[cp] = canonicalizeEncoding(name);
            });

          case 4:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));
  return _codepage.apply(this, arguments);
}

function charset(_x2) {
  return _charset.apply(this, arguments);
}

function _charset() {
  _charset = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee3(codepage) {
    var charset;
    return _regenerator.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return reg.query("HKCR/MIME/Database/Charset");

          case 2:
            charset = _context3.sent;
            Object.keys(charset).map(function (name) {
              var data = charset[name];
              var cp = data.Codepage || data.InternetEncoding;

              if (cp) {
                codepage[canonicalizeEncoding(name)] = cp;
              } else if (data.AliasForCharset) {
                return [name, data.AliasForCharset];
              }
            }).forEach(function (charset) {
              if (charset) {
                codepage[canonicalizeEncoding(charset[0])] = codepage[canonicalizeEncoding(charset[1])];
              }
            });

          case 4:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));
  return _charset.apply(this, arguments);
}

var magic = {
  "utf16le": 1200,
  "utf16be": 1201,
  "utf32le": 12000,
  "utf32be": 12001,
  "utf64le": 16969,
  "utf7": 65000,
  "utf8": 65001
};

function patch(database) {
  for (var encoding in magic) {
    var _codepage2 = magic[encoding];
    database.encoding[String(_codepage2)] = encoding;
    database.codepage[encoding] = _codepage2;
  }
}

function sort(database) {
  var encoding = {};
  Object.keys(database.encoding).sort(function (a, b) {
    return a - b;
  }).forEach(function (key) {
    encoding[key] = database.encoding[key];
  });
  database.encoding = encoding;
  var codepage = {};
  Object.keys(database.codepage).sort().sort(function (a, b) {
    return database.codepage[a] - database.codepage[b] || a.localeCompare(b);
  }).forEach(function (key) {
    codepage[key] = database.codepage[key];
  });
  database.codepage = codepage;
}

function database() {
  return _database.apply(this, arguments);
}

function _database() {
  _database = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee4() {
    var database;
    return _regenerator.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return identifiers();

          case 2:
            database = _context4.sent;
            _context4.next = 5;
            return Promise.all([codepage(database.encoding), charset(database.codepage)]);

          case 5:
            patch(database);
            sort(database);
            _context4.next = 9;
            return writeFile(require.resolve("./charset.json"), JSON.stringify(database, null, "\t") + "\n");

          case 9:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));
  return _database.apply(this, arguments);
}

module.exports = database;