"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var spawn = require("./spawn");

var aliasWin = require("alias-win");

var gitWin = require("git-win");

var path = require("path");

var fs = require("fs");

var os = require("os");

var promisify = require("util").promisify || require("util.promisify");

var binDir = path.join(__dirname, "../bin");

require("./env-value");

function getDefaultAliases() {
  var gitPath = function gitPath(file) {
    return path.join("%GIT_INSTALL_ROOT%", file);
  };

  var env = function env(cmds) {
    return cmds ? `env.exe ${cmds} $*` : "env.exe $*";
  };

  var shell = function shell(_shell) {
    return env(`"SHELL=${gitPath(_shell)}" ${path.basename(_shell, ".exe")}`);
  };

  var js = function js(_js) {
    return env(`node "${require.resolve(_js)}"`);
  };

  var sh = shell("/bin/sh");
  return {
    alias: js("alias-win/bin/alias"),
    unalias: js("alias-win/bin/unalias"),
    npx: env("npx"),
    npm: env("npm"),
    bash: shell("/bin/bash"),
    dash: shell("/bin/dash"),
    zsh: shell("/usr/bin/zsh"),
    sh,
    env: `if defined SHELL ( ${env()} ) else ( ${env(`"SHELL=${gitPath("bin/sh")}" env.exe`)} )`,
    "$SHELL": `if defined SHELL ( ${env("\"%SHELL%\"")} ) else ( ${sh} )`
  };
}

function set(key, value) {
  value = value || process.env[key];

  if (value) {
    console.log(`set "${key}=${value}"`);
  }
}

function login() {
  return _login.apply(this, arguments);
}

function _login() {
  _login = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee() {
    var envPath, HOME, env, rawAliases, aliases, name;
    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            envPath = gitWin.toWin32("/usr/bin/env.exe");

            if (process.env.SHELL) {
              set("PATH");
              console.log(`"${envPath}" "${process.env.SHELL}" --login`);
            }

            HOME = gitWin.toPosix(process.env.HOME);
            _context.next = 5;
            return spawn([envPath, "--ignore-environment", "HOME=" + HOME, process.env.SHELL || "/bin/bash", "--login", "-c", "env && alias -p"], {
              env: {
                MSYSTEM: gitWin.mingw.toUpperCase()
              },
              encoding: "utf8"
            });

          case 5:
            env = _context.sent;
            _context.next = 8;
            return aliasWin.get();

          case 8:
            rawAliases = _context.sent;
            aliases = Object.assign(getDefaultAliases(), rawAliases);
            env.split(/[\r\n]+/g).forEach(function (env) {
              if (!env || /^(?:!.+|_|(?:ORIGINAL|MSYSTEM|MINGW)(?:_.+)?|HOME|PWD|SHELL|TERM|PS1|SHLVL|SYSTEMROOT|WINDIR)=/i.test(env) || /=\/mingw\d+(?=\/|$)/.test(env)) {
                return;
              } else if (/^PATH=/i.test(env)) {
                env = env.slice(5);
                process.env.PATH = env.replace(new RegExp("(^|:)" + HOME + "(?=/|:|$)", "g"), "$1~");
                set("PATH");
                return;
              } else if (/^alias\s+(.*?)\s*=\s*(['"])?(.*)\2$/.test(env)) {
                var key = RegExp.$1,
                    value = RegExp.$3;
                aliases[key] = value.replace(/\s*$/, " $*");
                return;
              }

              console.log(`set "${env}"`);
            });

            for (name in rawAliases) {
              if (aliases[name] === rawAliases[name]) {
                delete aliases[name];
              }
            }

            _context.next = 14;
            return aliasWin.set(aliases);

          case 14:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  return _login.apply(this, arguments);
}

function npmrc() {
  return _npmrc.apply(this, arguments);
}

function _npmrc() {
  _npmrc = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee3() {
    var nodeDir, readFile, read, _read, npmConfig;

    return _regenerator.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _read = function _ref2() {
              _read = (0, _asyncToGenerator2.default)(
              /*#__PURE__*/
              _regenerator.default.mark(function _callee2(file) {
                var ini, contents;
                return _regenerator.default.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        _context2.prev = 0;
                        ini = require(path.join(nodeDir, "node_modules/npm/node_modules/ini/ini.js"));
                        _context2.next = 4;
                        return readFile(file, "utf8");

                      case 4:
                        contents = _context2.sent;
                        return _context2.abrupt("return", ini.parse(contents));

                      case 8:
                        _context2.prev = 8;
                        _context2.t0 = _context2["catch"](0);
                        return _context2.abrupt("return", {});

                      case 11:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, _callee2, this, [[0, 8]]);
              }));
              return _read.apply(this, arguments);
            };

            read = function _ref(_x) {
              return _read.apply(this, arguments);
            };

            nodeDir = path.dirname(process.execPath);
            readFile = promisify(fs.readFile);
            npmConfig = {
              prefix: process.env.npm_config_prefix || path.join(process.env.APPDATA, "npm"),
              shell: process.env.npm_config_shell
            };
            _context3.next = 7;
            return Promise.all([path.join(npmConfig.prefix, "etc/npmrc"), path.join(os.homedir(), ".npmrc"), ".npmrc"].map(read)).then(function (config) {
              Object.assign.apply(Object, [npmConfig].concat((0, _toConsumableArray2.default)(config)));
            });

          case 7:
            if (npmConfig.shell && !/^(?:.*\\)?cmd(?:\.exe)?$/i.test(npmConfig.shell)) {
              process.env.SHELL = npmConfig.shell;
            }

          case 8:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));
  return _npmrc.apply(this, arguments);
}

delete process.env.GIT_BASH_SHELL_INIT;
process.env.PATH = binDir + ";" + process.env.PATH;
npmrc().then(login).then(function () {
  set("SHELL");

  if (process.env.ProgramData) {
    set("CMDER_ROOT", path.join(process.env.ProgramData, "Cmder"));
  }

  set("GIT_INSTALL_ROOT", gitWin.root);
});