"use strict";

var gitWin = require("git-win");

var path = require("path");

var os = require("os");

var binDir = path.join(__dirname, "../bin");
var env = process.env;
var posixPath;

function getShell(shell) {
  if (shell) {
    shell = gitWin.resolve(shell).replace(/^(\/.*?)(?:\.(?:exe|cmd|bat|com))?$/, "$1");
  }

  return shell;
}

function getHome(env) {
  return env.HOME || (env.HOME = os.homedir());
}

function pathSort(dir) {
  if (!dir) {
    return 0xff;
  } else if (binDir === dir) {
    return -0xff;
  }

  return 0;
}

function setPath(env, value) {
  if (!/;/.test(value) && value.split(/:+/g).every(function (dir) {
    return /^(?:\.$|~?\/)/.test(dir);
  })) {
    posixPath = Array.from(new Set(value.split(/:/g))).filter(Boolean);
  } else {
    env.Path = value;
  }
}

function getPath(env) {
  var Path = env.Path.split(/;+/g);

  if (posixPath) {
    Path = Path.filter(function (dir) {
      return posixPath.indexOf(gitWin.resolve(dir)) < 0;
    });
  } else {
    Path = Array.from(new Set(Path)).sort(function (a, b) {
      return pathSort(a) - pathSort(b);
    });
  }

  Path = Path.join(";");

  if (posixPath) {
    var ending;
    var newPath = posixPath.map(function (dir) {
      if (dir === "." && ending == null) {
        return Path.replace(/[\s;]*$/, function (s) {
          ending = s;
          return "";
        });
      } else {
        return gitWin.toWin32(dir).replace(/^%HOME%/, function () {
          return getHome(env);
        });
      }
    }).join(";");

    if (ending == null) {
      Path = newPath + ";" + Path;
    } else {
      Path = newPath + ending;
    }
  }

  return Path;
}

var envHandler = {
  get: function get(env, prop) {
    if (typeof prop === "string") {
      if (/^PATH$/i.test(prop)) {
        return getPath(env);
      } else if (/^ComSpec$/i.test(prop)) {
        return getShell(env.SHELL) || env[prop];
      } else if (/^npm_config(?:_\w+)*_shell$/i.test(prop)) {
        return getShell(env[prop] || env.SHELL);
      } else if (/^SHELL$/i.test(prop)) {
        return getShell(env.SHELL);
      } else if (/^HOME$/i.test(prop)) {
        return getHome(env);
      } else if (/^APPDATA$/i.test(prop)) {
        return env[prop] || path.join(os.homedir(), "AppData/Roaming");
      } else if (/^(?:SystemRoot|windir)$/i.test(prop)) {
        return env[prop] || path.join(env.SystemDrive || "C:", "Windows");
      }
    }

    return env[prop];
  },
  set: function set(env, prop, value) {
    if (typeof prop === "string" && /^PATH$/i.test(prop)) {
      setPath(env, value);
    } else {
      env[prop] = value;
    }

    return true;
  }
};
process.env = new Proxy(env, envHandler);
module.exports = env;