"use strict";

var childProcess = require("child_process");

function stdio(stdio, options) {
  var arrStdio = [];

  if (stdio) {
    stdio.on("data", function (data) {
      arrStdio.push(data);
    });
  }

  return function () {
    stdio = Buffer.concat(arrStdio);

    if (options.encoding && options.encoding !== "buffer") {
      stdio = stdio.toString(options.encoding);
    }

    return stdio;
  };
}

function spawn(commands) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var child = childProcess.spawn(commands.shift(), commands.filter(Boolean), options);

  if (options.echo) {
    console.log(">", child.spawnargs.join(" "));
  }

  var getStdout = stdio(child.stdout, options);
  var getStderr = stdio(child.stderr, options);

  if (child.stdin && options.input) {
    if (options.input.pipe) {
      options.input.pipe(child.stdin);
    } else {
      child.stdin.end(options.input);
    }
  }

  return new Promise(function (resolve, reject) {
    child.on("error", reject);
    child.on("exit", function (code) {
      if (code) {
        var stderr = getStderr();
        var err = new Error("> " + child.spawnargs.join(" ") + "\n" + (String(stderr) || `child exited with code ${code}`));
        err.stderr = stderr;
        err.stdout = getStdout();
        err.errno = code;
        reject(err);
      } else {
        resolve(getStdout());
      }
    });
  });
}

module.exports = spawn;