"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _marked =
/*#__PURE__*/
_regenerator.default.mark(filePathList),
    _marked2 =
/*#__PURE__*/
_regenerator.default.mark(fileExtList);

var getEnvValue = require("./get-env-value");

var gitWin = require("git-win");

var LRU = require("lru-cache");

var path = require("path");

var fs = require("fs");

var fsCache = new LRU({
  max: 500,
  maxAge: 30 * 1000
}); // Cache just for 30sec

var fsStorage = new Map();

function win32Path(paths, options) {
  return gitWin.toWin32.apply(gitWin, (0, _toConsumableArray2.default)(paths)).replace(/%(.+?)%/g, function (s, key) {
    return getEnvValue(key, options) || s;
  });
}

function stat(strPath) {
  var dirCache = /^[A-Z]:\\(?:Program\s+Files(?:\s+\(x\d+\))?|Windows)(?=\\|$)/i.test(strPath) || gitWin.fixPosixRoot(strPath) ? fsStorage : fsCache;

  if (dirCache.has(strPath)) {
    return dirCache.get(strPath);
  }

  var stats;

  try {
    stats = fs.statSync(strPath);
  } catch (ex) {
    stats = null;
  }

  dirCache.set(strPath, stats);
  return stats;
}

function readDir(dir) {
  var stats = stat(dir);

  if (!stats || !stats.isDirectory()) {
    return;
  }

  if (stats.items) {
    return stats.items;
  }

  var files;

  try {
    files = fs.readdirSync(dir);
  } catch (ex) {//
  }

  var items = new Map(files && files.map(function (filename) {
    return [filename.toLowerCase(), filename];
  }));
  stats.items = items;
  return items;
}

function filePathList(file, options) {
  var cwd, Path, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, dir;

  return _regenerator.default.wrap(function filePathList$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          cwd = options && options.cwd && path.resolve(options.cwd) || process.cwd();
          _context.next = 3;
          return win32Path([cwd, file], options);

        case 3:
          if (!/[/\\]/.test(file)) {
            _context.next = 9;
            break;
          }

          if (!path.posix.isAbsolute(file)) {
            _context.next = 7;
            break;
          }

          _context.next = 7;
          return path.resolve(cwd, file);

        case 7:
          _context.next = 37;
          break;

        case 9:
          Path = getEnvValue("PATH", options);

          if (!Path) {
            _context.next = 37;
            break;
          }

          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context.prev = 14;
          _iterator = Path.split(/(?:\s*;\s*)+/g).filter(Boolean)[Symbol.iterator]();

        case 16:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context.next = 23;
            break;
          }

          dir = _step.value;
          _context.next = 20;
          return win32Path([cwd, dir, file], options);

        case 20:
          _iteratorNormalCompletion = true;
          _context.next = 16;
          break;

        case 23:
          _context.next = 29;
          break;

        case 25:
          _context.prev = 25;
          _context.t0 = _context["catch"](14);
          _didIteratorError = true;
          _iteratorError = _context.t0;

        case 29:
          _context.prev = 29;
          _context.prev = 30;

          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }

        case 32:
          _context.prev = 32;

          if (!_didIteratorError) {
            _context.next = 35;
            break;
          }

          throw _iteratorError;

        case 35:
          return _context.finish(32);

        case 36:
          return _context.finish(29);

        case 37:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this, [[14, 25, 29, 37], [30,, 32, 36]]);
}

function fileExtList(file, options) {
  var pathExt, ext, has, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, filePath, dirname, strDir, files, basename, exts, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, extName;

  return _regenerator.default.wrap(function fileExtList$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          ext = function _ref() {
            if (!pathExt) {
              pathExt = getEnvValue("PATHEXT", options);
              pathExt = pathExt ? pathExt.toLowerCase().split(/\s*;\s*/g) : [];
            }

            return pathExt;
          };

          has = {};
          _iteratorNormalCompletion2 = true;
          _didIteratorError2 = false;
          _iteratorError2 = undefined;
          _context2.prev = 5;
          _iterator2 = filePathList(file, options)[Symbol.iterator]();

        case 7:
          if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
            _context2.next = 53;
            break;
          }

          filePath = _step2.value;
          dirname = path.dirname(filePath);
          strDir = dirname.toLowerCase();

          if (!has[strDir]) {
            _context2.next = 13;
            break;
          }

          return _context2.abrupt("continue", 50);

        case 13:
          has[strDir] = true;
          files = readDir(strDir);

          if (files) {
            _context2.next = 17;
            break;
          }

          return _context2.abrupt("continue", 50);

        case 17:
          basename = path.basename(filePath).toLowerCase();
          _context2.next = 20;
          return [files, dirname, basename];

        case 20:
          exts = ext();

          if (!(gitWin.fixPosixRoot(filePath) && exts.indexOf(".exe") < 0)) {
            _context2.next = 24;
            break;
          }

          _context2.next = 24;
          return [files, dirname, basename + ".exe"];

        case 24:
          _iteratorNormalCompletion3 = true;
          _didIteratorError3 = false;
          _iteratorError3 = undefined;
          _context2.prev = 27;
          _iterator3 = exts[Symbol.iterator]();

        case 29:
          if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
            _context2.next = 36;
            break;
          }

          extName = _step3.value;
          _context2.next = 33;
          return [files, dirname, basename + extName];

        case 33:
          _iteratorNormalCompletion3 = true;
          _context2.next = 29;
          break;

        case 36:
          _context2.next = 42;
          break;

        case 38:
          _context2.prev = 38;
          _context2.t0 = _context2["catch"](27);
          _didIteratorError3 = true;
          _iteratorError3 = _context2.t0;

        case 42:
          _context2.prev = 42;
          _context2.prev = 43;

          if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
            _iterator3.return();
          }

        case 45:
          _context2.prev = 45;

          if (!_didIteratorError3) {
            _context2.next = 48;
            break;
          }

          throw _iteratorError3;

        case 48:
          return _context2.finish(45);

        case 49:
          return _context2.finish(42);

        case 50:
          _iteratorNormalCompletion2 = true;
          _context2.next = 7;
          break;

        case 53:
          _context2.next = 59;
          break;

        case 55:
          _context2.prev = 55;
          _context2.t1 = _context2["catch"](5);
          _didIteratorError2 = true;
          _iteratorError2 = _context2.t1;

        case 59:
          _context2.prev = 59;
          _context2.prev = 60;

          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }

        case 62:
          _context2.prev = 62;

          if (!_didIteratorError2) {
            _context2.next = 65;
            break;
          }

          throw _iteratorError2;

        case 65:
          return _context2.finish(62);

        case 66:
          return _context2.finish(59);

        case 67:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this, [[5, 55, 59, 67], [27, 38, 42, 50], [43,, 45, 49], [60,, 62, 66]]);
}

function which(file, options) {
  var _iteratorNormalCompletion4 = true;
  var _didIteratorError4 = false;
  var _iteratorError4 = undefined;

  try {
    for (var _iterator4 = fileExtList(file, options)[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
      var _step4$value = (0, _slicedToArray2.default)(_step4.value, 3),
          files = _step4$value[0],
          dirname = _step4$value[1],
          basename = _step4$value[2];

      var realName = files.get(basename);

      if (realName) {
        realName = path.join(dirname, realName);

        if (/\.(?:exe|cmd|bat|com)$/i.test(basename)) {
          return realName;
        }

        var stats = stat(realName.toLowerCase());

        if (stats && stats.isFile()) {
          return realName;
        }
      }
    }
  } catch (err) {
    _didIteratorError4 = true;
    _iteratorError4 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion4 && _iterator4.return != null) {
        _iterator4.return();
      }
    } finally {
      if (_didIteratorError4) {
        throw _iteratorError4;
      }
    }
  }
}

module.exports = which;